//------------------------------------------------------------------------------
//	Math utilities
//
//	Author: Grzegorz Skowron & Milosz Zdybal
//------------------------------------------------------------------------------

#ifndef __COSKEL_UTILS__
#define __COSKEL_UTILS__

#include <time.h>
#include <unistd.h>
#include "coskel_confparser.h"

namespace coskel {

//------------------------------------------------------------------------------
//	Global functions declarations
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	Structures
//------------------------------------------------------------------------------

struct EXECTIME
{
	time_t start;
	time_t end;
};

// Changes the specified complex number string into a complex<double> object
std::complex<double> string_to_complex(const std::string &str);

// Changes the specified complex<double> object into a string
std::string complex_to_string(const std::complex<double> &cnum);

// Prepares some header info about the program
std::string prepare_info(int argc,char *argv[],const std::string &title,const std::string &author,const confparser &cp,struct EXECTIME *et);

void tic(struct EXECTIME *et); // Sets the beggining time
void toc(struct EXECTIME *et); // Sets the ending time

//------------------------------------------------------------------------------
//	Global functions definitions
//------------------------------------------------------------------------------

std::complex<double> string_to_complex(const std::string &str)
{
	std::string tmp;
	int sign_state = 1;
	double re = 0.0,im = 0.0;
	
	for(unsigned int i=0;i<str.length();i++)
	{
		switch(str[i])
		{
			case '+':
				tmp += " + "; break;
			case '-':
				tmp += " - "; break;
			default:
				tmp += str[i]; break;
		}
	}
	
	std::istringstream tmp_stream(tmp);
	size_t pos;
	
	while(tmp_stream >> tmp)
	{
		if(tmp == "+") sign_state = 1;
		else if(tmp == "-")	sign_state = -1;
		else
		{
			pos = tmp.find("i");
			if(pos == std::string::npos) pos = tmp.find("I");
			if(pos != std::string::npos)
			{
				tmp.erase(pos,1);
				if(tmp.empty()) im = sign_state * 1.0;
				else im = sign_state * atof(tmp.c_str());
			}
			else re = sign_state * atof(tmp.c_str());
		}
	}
	return std::complex<double>(re,im);
}
//------------------------------------------------------------------------------
std::string complex_to_string(const std::complex<double> &cnum)
{
	std::string ret;
	std::stringstream str(std::stringstream::in|std::stringstream::out);
	
	str << cnum.real();
	if(cnum.imag() >= 0) str << "+";
	str << cnum.imag() << "i";
	str >> ret;
	
	return ret;
}
//------------------------------------------------------------------------------
std::string prepare_info(int argc,char *argv[],const std::string &title,const std::string &author,confparser *cp,const struct EXECTIME *et)
{	
	std::string line="";
	char hname[256];
	
	char buff[256];
	time_t t;
	struct tm tmp;
 
    if(!title.empty()) 
    	line += "## "+title+"\n";
    
    if(author.empty()) 
    	line += "## "+std::string(argv[0])+" [version: "+__DATE__+" "+__TIME__+"]\n";
	else 
    	line += "## "+std::string(argv[0])+" [version: "+__DATE__+" "+__TIME__+" author: "+author+"]\n";
	
	line += "## command-line: ";

	for(int i=0;i<argc;i++) 
		line += std::string(argv[i]) + " ";
		
	line += "\n";

	if(cp != NULL)
	{
		line += "##\n## BEGIN CONFIG:\n";
		line += cp->get_info();
		line += "## END CONFIG:\n";
	}
	
	if(et != NULL)
	{
		gethostname(hname,255);
		
		line += "##\n";
		
		localtime_r(&et->start,&tmp);  
		strftime(buff,256, "%G-%m-%dT%H:%M:%S%z",&tmp);
		
		line += "## Start ("+std::string(hname)+") "+std::string(buff)+"\n";
		
		localtime_r(&et->end,&tmp);
		strftime(buff,256, "%G-%m-%dT%H:%M:%S%z",&tmp);	
		
		line += "## Stop ("+std::string(hname)+") "+std::string(buff)+"\n\n";    
	}
	
	return line;
}

//------------------------------------------------------------------------------
void tic(struct EXECTIME &et)
{
	et.start = time(NULL); 
}
//------------------------------------------------------------------------------
void toc(struct EXECTIME &et)
{
	et.end = time(NULL); 	
}

};

#endif
