#include <typeinfo>
#include <iostream>
#include <valarray>
#include <complex>
#include <vector>
#include <fstream>

#include "coskel_utils.h"
#include "coskel_confparser.h"
#include "coskel_clparser.h"
#include "coskel_matrix.h"

using namespace std;

int main(int argc,char *argv[])
{	
	//--------------------------------------------------------------------------
	// Here we have all important declarations
	//--------------------------------------------------------------------------
	
		coskel::confparser confp;
		coskel::clparser commlp;
		coskel::matrix<int> mat;
		
	//--------------------------------------------------------------------------
	// Now we should define command line arguments nd read config file
	//--------------------------------------------------------------------------
	
		// It doesn't matter but let's read the config file first :). command line args are next
		confp.load_file("data.conf");
		
		// For example two noname arguments... Input and output files
		commlp.register_noname_arg("infile","Input file");
		commlp.register_noname_arg("outfile","Output file");
		
		// And one switch: is_parametric = false AND is_required = false
		commlp.register_arg("hw","Display hello world instead of everything else",false,false);
		
		// Don't forget to parse the command-line!
		if(!commlp.parse(argc,argv))
		{
			cout << commlp.get_info(argv[0]); // We messed up something so please display help info
			return 1;
		}
		
	//--------------------------------------------------------------------------
	// Ok we have everything done. Now just matrices to read and perform calculations
	// We can check other things as well :)
	//--------------------------------------------------------------------------
	
		if(commlp.check_presence("hw"))
		{
			cout << "Hello world! :D" << endl;
			return 0;
		}
	
		mat.load_from_file("data.in"); // let's load a whole file
		
		// HERE YOU CAN PUT YOUR COMPUTING ROUTINES. We will add 1 to the all elements
		for(int i=0;i<mat.rows();i++)
		for(int j=0;j<mat.cols();j++)
			mat[i][j] += 1;
	
	//--------------------------------------------------------------------------
	// Now it's time to save everything right :D
	//--------------------------------------------------------------------------	
	
		mat.save_to_file("data.out",prepare_info(argc,argv,"Demo","Grzegorz Skowron",&confp,NULL));
	
	return 0;
}
